@@ -285,3 +285,18 @@ class CvtType:
 def mod_minabs(x, m):
     a = x % m
     return a if a < m / 2 else a - m
+
+
+class classproperty(property):
+    """ python - Using property() on classmethods - Stack Overflow
+        https://stackoverflow.com/questions/128573/using-property-on-classmethods
+    """
+
+    def __get__(self, obj, objtype=None):
+        return super(classproperty, self).__get__(objtype)
+
+    def __set__(self, obj, value):
+        super(classproperty, self).__set__(type(obj), value)
+
+    def __delete__(self, obj):
+        super(classproperty, self).__delete__(type(obj))
